/*
 * Miscellaneous utility functions
 *
 * Michael Pearson <alcaron@ozemail.com.au>
 * Student Number:	s0383042
 * Last Revision: 	25/03/99
 *
 * Known Bugs:		Zip.
 *
 */

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<stdarg.h>
#include<ctype.h>
#include"utility.h"

/* #define UTILITY_DEBUG 0 */
#define MAXOPTIONS 10

static char *runtimeName;
static int verbosity = 0;

/*
 * gets a string from the keyboard, with a prompt
 */
char *getString(const char * format, ...) {
	char inputbuff[100];
	va_list args;
	
	va_start(args, format);
	vprintf(format, args);
	va_end(args);
	fgets(inputbuff, 99, stdin);
	inputbuff[strlen(inputbuff) - 1] = '\0';   /* get rid of the newline */
	return(safeStrdup(inputbuff));
}

/*
 * Same as a simple malloc, but does error checking.
 */
void *safeMalloc(size_t size) {
	void *ptr;

	ptr = malloc(size);

	/* If malloc returned NULL, die */
	if(!ptr) {
		fatalError(1, "Could not allocate block of %d size. Possibly out of"
		           "memory.", size);
	}

	return ptr;
}

/*
 * Same as a simple realloc, but does error checking
 */
void *safeRealloc(void *block, size_t size) {
	void *ptr;

	ptr = realloc(block, size);

	/* If realloc returned NULL, die */
	if(!ptr) {
		fatalError(1, "Could not realloc block at 0x%x %d size. Possibly out of"
		           "memory.", (int) block, size);
	}

	return ptr;
}

/*
 * Same as a simple strdup, but does error checking
 */
char *safeStrdup(const char *s) {
	char *ptr;

	ptr = strdup(s);

	/* If strdup returned NULL, die */
	if(!ptr) {
		fatalError(1, "Could not duplicate string at 0x%s.", s);
	}

	return ptr;
}

/*
 * Same as a simple strndup, but does error checking
 */
char *safeStrndup(const char *s, size_t size) {
    char *ptr;

    ptr = (char *) strndup(s, size);

    /* If strdup returned NULL, die */
    if(!ptr) {
		fatalError(1, "Could not duplicate string at 0x%s.", s);
    }

    return ptr;
}

/*
 * gets a double value from the keyboard
 */
double getDouble (const char *prompt) {
	double result;
	char *buffer;

	buffer = getString(prompt);
	stripString(buffer);
	result = atof(buffer);
	free(buffer);
	return result;
}

/*
 * gets an int value from the keyboard
 */
int getInt (const char *prompt) {
	int result;
	char *buffer;

	buffer = getString(prompt);
	stripString(buffer);
	result = atoi(buffer);
	free(buffer);
	return result;
}

/*
 * Appends a character to the end of a string
 */
void appendChar(char *dest, char src) {
	dest = dest + strlen(dest);
	*dest++ = src;
	*dest = '\0';	/* Terminate string */
}

/*
 * gets one of a specified set of options. An error is displayed and the
 * options repeated if an invalid choice is entered.
 */
char getOption(const char *prompt, const char *options) {
	char choice, *choicebuffer, *errorstring;
	int i, numoptions;

	numoptions = strlen(options);

	/* Make our error string */
	errorstring = safeMalloc(strlen("Please enter  or  .\n") + numoptions * 3 + 1);
	strcpy(errorstring, "Please enter ");
	for(i=0; i<numoptions - 2; i++) {
		appendChar(errorstring, options[i]);
		strcat(errorstring, ", ");
	}
	appendChar(errorstring, options[i]);
	strcat(errorstring, " or ");
	appendChar(errorstring, options[i + 1]);
	strcat(errorstring, ".\n");

	/* Do the actual work. The for loop acts as a sentinel */
	do {
		choicebuffer = getString(prompt);
		choice = *choicebuffer;
		choice = toupper(choice);
		free(choicebuffer);
		printf("\n");
		for(i=0; i<numoptions; i++) {
			if(options[i] == choice)
				break;
		}
		if(i == numoptions)
			puts(errorstring);
	} while(i == numoptions);

	free(errorstring);
	return choice;
}

/*
 * Makes each character in the string uppercase
 */
void strUpper(char *string) {
	while(*string)
		*string++ = toupper(*string);
}

/*
 * Makes each character in the string lowercase
 */
void strLower(char *string) {
	while(*string)
		*string++ = tolower(*string);
}

/*
 * gets a yes or no answer from a question.
 * Returns TRUE for yes, naturally.
 */
int getYesNo(const char *prompt) {
	int result = -1;
	char *choice;

	/* Repeat this until the user enters yes or no */
	do {
		choice = getString(prompt);
		strUpper(choice);

		if(!strcmp(choice, "Y") || !strcmp(choice, "YES")) {
			result = 1;
		} else if(!strcmp(choice, "N") || !strcmp(choice, "NO")) {
			result = 0;
		} else {
			printf("Please enter either Yes or No.\n\n");
		}

		free(choice);
	} while(result < 0);
	return result;
}

/*
 * strips a string of all non digits (period is counted as a digit)
 */
void stripString(char *string) {
	char *current;

	current = string;
	while(*string) {
		if(isdigit(*string) || *string == '.')
			*current++ = *string;
		string++;
	}
	*current = '\0';
}

/*
 * sets the current verbosity. Atm, this is just 0 (none), 1 (on), or 
 * -1 (suppress warnings)
 */
void setVerbose(int verbosity) {
	verbosity = verbosity;
}

/*
 * Return the current verbosity
 */
int getVerbose(void) {
	return verbosity;
}

/*
 * sets the runtimeName.
 * Potential Memory leak here, but I don't think anybody's going to need
 * to set the run time name more than once...
 */
void setRuntimeName(const char *name) {
	runtimeName = safeStrdup(name);
}

/*
 * gets the runtimeName. Don't mess with this string
 */
const char *getRuntimeName(void) {
	return runtimeName;
}

/*
 * Prints the error and dies horribly.
 */
void fatalError(int errorcode, const char *format, ...) {
	va_list args;
	
	va_start(args, format);
	fprintf(stderr, "\n%s: ERROR : ", runtimeName);
	vfprintf(stderr, format, args);
	fprintf(stderr, "\n");
	va_end(args);
	
	exit(errorcode);
}

/*
 * Prints a warning message if verbosity is 0 or over
 */
void warning(const char *format, ...) {
	va_list args;
	
	if(verbosity >= 0) {
		va_start(args, format);
		fprintf(stderr, "%s: WARNING : ", runtimeName);
		vfprintf(stderr, format, args);
		fprintf(stderr, "\n");
		va_end(args);
	}
}

/*
 * Prints a message if verbosity is > 0
 */
void verbose(const char *format, ...) {
	va_list args;
	
	if(verbosity > 0) {
		va_start(args, format);
		printf("- ");
		vprintf(format, args);
		printf("\n");
		va_end(args);
	}
}

char *ptrStrDup(char *start, char *end)
{
    char *out, *ptr;

	out = safeMalloc((size_t) end - (size_t) start + 1);

    ptr = out;
    while (start < end)
        *ptr++ = *start++;

    *ptr = '\0';

	return out;
}
	
/*** A Wizard in a Blizzard and a Mystical Machine Gun! ***/
